@@ -1,7 +1,7 @@
-/* @(#)storage.c	1.37 00/12/24 Copyright 1984 J. Schilling */
+/* @(#)storage.c	1.41 01/06/20 Copyright 1984 J. Schilling */
 #ifndef lint
 static	char sccsid[] =
-	"@(#)storage.c	1.37 00/12/24 Copyright 1984 J. Schilling";
+	"@(#)storage.c	1.41 01/06/20 Copyright 1984 J. Schilling";
 #endif
 /*
  *	Storage management based on the paged virtual memory
@@ -79,8 +79,8 @@ extern	int	errno;
 #endif
 #endif
 
-#define	gaplink(p)	((headr_t *)((p)->gaplink))
-#define	winlink(p)	((headr_t *)((p)->winlink))
+#define	wp_gaplink(p)	((headr_t *)((p)->gaplink))
+#define	wp_winlink(p)	((headr_t *)((p)->winlink))
 
 EXPORT	void	insert		__PR((ewin_t *wp, Uchar* str, long size));
 EXPORT	void	delete		__PR((ewin_t *wp, epos_t size));
@@ -140,10 +140,10 @@ insert(wp, str, size)
 		/*
 		 * There is no leading gap after a movegap() call.
 		 */
-		amount = min(n, BUFFERSIZE - gaplink(wp)->size);
-		to = gaplink(wp)->cont + gaplink(wp)->size;
-		gaplink(wp)->flags |= MODIFIED;
-		gaplink(wp)->size += amount;
+		amount = min(n, BUFFERSIZE - wp_gaplink(wp)->size);
+		to = wp_gaplink(wp)->cont + wp_gaplink(wp)->size;
+		wp_gaplink(wp)->flags |= MODIFIED;
+		wp_gaplink(wp)->size += amount;
 		n -= amount;
 		if (amount > 16) {
 			movebytes(C str, C to, amount);
@@ -153,7 +153,7 @@ insert(wp, str, size)
 				*to++ = *str++;
 		}
 		if (n)
-			wp->gaplink = addbuffer(wp, gaplink(wp));
+			wp->gaplink = addbuffer(wp, wp_gaplink(wp));
 	}
 	wp->eof += size;
 	wp->gap += size;
@@ -187,7 +187,7 @@ delete(wp, size)
 	movegap(wp, wp->dot);
 
 	while (n > 0) {
-		if ((next = gaplink(wp)->next) == NULL)	/* Paranoia */
+		if ((next = wp_gaplink(wp)->next) == NULL)	/* Paranoia */
 			break;
 #ifdef	FASTPOS
 		if (next == wp->winlink)
@@ -246,18 +246,18 @@ rubout(wp, size)
 	movegap(wp, wp->dot);
 
 	while (n > 0) {
-		if (! gaplink(wp)->next)	/* Paranoia */
+		if (! wp_gaplink(wp)->next)	/* Paranoia */
 			break;
 #ifdef	FASTPOS
-		if (gaplink(wp) == wp->winlink)
+		if (wp_gaplink(wp) == wp->winlink)
 			clearwpos(wp);
 #endif
-		amount = min(n, gaplink(wp)->size);
+		amount = min(n, wp_gaplink(wp)->size);
 		n -= amount;
-		gaplink(wp)->size -= amount;
+		wp_gaplink(wp)->size -= amount;
 		if (n)
-			wp->gaplink = deletebuffer(wp, gaplink(wp));
-/*		readybuffer(wp, gaplink(wp));*/
+			wp->gaplink = deletebuffer(wp, wp_gaplink(wp));
+/*		readybuffer(wp, wp_gaplink(wp));*/
 	}
 	wp->dot -= size;
 	wp->eof -= size;
@@ -283,6 +283,9 @@ loadfile(wp, filename, newdefault)
 	FILE	*infile;
 	Uchar	str[BUFSIZ < 8192 ? 8192 : BUFSIZ];
 	long	size;
+#ifdef	IS_CYGWIN
+	BOOL	retry = FALSE;
+#endif
 
 	if (newdefault) {
 		wp->eflags &= ~(FREADONLY|FNOLOCK);
@@ -291,7 +294,7 @@ loadfile(wp, filename, newdefault)
 		wp->curfd = -1;
 		if (wp->curfp != NULL)
 			fclose(wp->curfp);
-		if (!ReadOnly)
+		if (!ReadOnly && writable(filename))
 			lockfile(wp, C filename);
 	}
 	if ((infile = fileopen(C filename, "rub")) == (FILE *) NULL) {
@@ -307,8 +310,48 @@ loadfile(wp, filename, newdefault)
 		return (FALSE);
 	}
 	file_raise(infile, FALSE);
-	while ((size = readsyserr(wp, infile, str, sizeof str, UC "FILE")) > 0)
+#ifdef	IS_CYGWIN
+again:
+#endif
+	while ((size = readsyserr(wp, infile, str, sizeof str, UC "FILE")) > 0) {
 		insert(wp, str, size);
+	}
+	if (size < 0) {
+#ifdef	IS_CYGWIN
+		/*
+		 * On Cygwin, we cannot lock correctly.
+		 * Locks are on a fd base not on a process base.
+		 * In addition, we may not even read from a diffrerent fd
+		 * if we wold a write lock on another fd.
+		 */
+		if (newdefault && !retry && wp->eof == 0 &&
+		    wp->curfd >= 0 && geterrno() == EACCES) {
+			/*
+			 * If read did not work and we did nothing
+			 * until now, this is a Cygwin locking problem
+			 * and we must destroy the locking.
+			 */
+			close(wp->curfd);
+			wp->curfd = -1;
+			retry = TRUE;
+			goto again;
+		}
+#endif
+		fclose(infile);
+		return (FALSE);
+	}
+#ifdef	IS_CYGWIN
+	if (newdefault && !retry && wp->curfd >= 0) {
+		write_errno(wp, "LOCK WORKED!");
+		sleep(2);
+	}
+#ifdef	NONOONO
+	if (newdefault && wp->curfd < 0) {
+		if (!ReadOnly && writable(C filename))
+			lockfile(wp, C filename);
+	}
+#endif
+#endif
 	if (wp->curfd >= 0) {
 		/*
 		 * Due to a conceptional bug of lockf()/fcntl(f, F_SETLK)
@@ -680,17 +723,17 @@ movegap(wp, pos)
 	epos_t	pos;
 {
 #ifdef	DEBUG
-	cdbg("movegap(%lld) gaplink: %p", (Llong)pos, (void *)gaplink(wp));
+	cdbg("movegap(%lld) gaplink: %p", (Llong)pos, (void *)wp_gaplink(wp));
 #endif
-	if (pos == wp->gap && gaplink(wp) != (headr_t *)0)
+	if (pos == wp->gap && wp_gaplink(wp) != (headr_t *)0)
 		return;
-	if (gaplink(wp))
-		compressbuffer(wp, gaplink(wp));
+	if (wp_gaplink(wp))
+		compressbuffer(wp, wp_gaplink(wp));
 	findpos(wp, wp->gap = pos, (headr_t **)&wp->gaplink, &wp->gapoff);
-	if (gaplink(wp)->buf != gaplink(wp)->cont)
-		compressbuffer(wp, gaplink(wp));
-	if (gaplink(wp)->size > wp->gapoff)
-		splitbuffer(wp, gaplink(wp), wp->gapoff);
+	if (wp_gaplink(wp)->buf != wp_gaplink(wp)->cont)
+		compressbuffer(wp, wp_gaplink(wp));
+	if (wp_gaplink(wp)->size > wp->gapoff)
+		splitbuffer(wp, wp_gaplink(wp), wp->gapoff);
 }
 
 /*
@@ -852,7 +895,7 @@ findpos(wp, pos, returnlinkp, returnoff)
 				(void *)wp->winlink,
 				(Llong)wp->winpos, (Llong)wp->winoff,
 				(Llong)SAVE, (Llong)wp->eof);
-			error("findpos: winlink->next: %p\n", (void *)winlink(wp)->next);
+			error("findpos: winlink->next: %p\n", (void *)wp_winlink(wp)->next);
 			if (linkp)
 				error("linkp: %p linkp->flags = %X\n", (void *)linkp, linkp->flags);
 #endif
